@@ -1,48 +1,36 @@
 /*
     This file is part of KDE.
 
-    Copyright (c) 2008 Cornelius Schumacher <schumacher@kde.org>
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) version 3, or any
-    later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), which shall
-    act as a proxy defined in Section 6 of version 3 of the license.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-FileCopyrightText: 2008 Cornelius Schumacher <schumacher@kde.org>
 
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
 #include "atticabasejob.h"
 
-#include <QDebug>
-#include <QTimer>
-#include <QNetworkReply>
 #include <QAuthenticator>
+#include <QDebug>
+#include <QNetworkReply>
+#include <QTimer>
 
+#include "platformdependent.h"
 #include <attica_debug.h>
 #include <atticautils.h>
-#include "platformdependent.h"
 
 using namespace Attica;
 
-class BaseJob::Private
+class Q_DECL_HIDDEN BaseJob::Private
 {
 public:
     Metadata m_metadata;
     PlatformDependent *m_internals;
     QNetworkReply *m_reply;
+    bool aborted{false};
+    bool started = false;
 
     Private(PlatformDependent *internals)
-        : m_internals(internals), m_reply(nullptr)
+        : m_internals(internals)
+        , m_reply(nullptr)
     {
     }
 
@@ -52,11 +40,11 @@ public:
             return false;
         }
 
-        int httpStatusCode = m_reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-        if (httpStatusCode == 301 || // Moved Permanently
-                httpStatusCode == 302 || // Found
-                httpStatusCode == 303 || // See Other
-                httpStatusCode == 307) { // Temporary Redirect
+        const int httpStatusCode = m_reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+        if (httpStatusCode == 301 // Moved Permanently
+            || httpStatusCode == 302 // Found
+            || httpStatusCode == 303 // See Other
+            || httpStatusCode == 307) { // Temporary Redirect
             QNetworkRequest request = m_reply->request();
             QUrl redirectUrl(m_reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl());
             if (redirectUrl.isRelative()) {
@@ -90,12 +78,12 @@ void BaseJob::dataFinished()
         return;
     }
 
-    bool error = (d->m_reply->error() != QNetworkReply::NoError);
+    bool error = d->m_reply->error() != QNetworkReply::NoError && d->m_reply->error() != QNetworkReply::OperationCanceledError;
 
     // handle redirections automatically
     QUrl newUrl;
     if (d->redirection(newUrl)) {
-        //qCDebug(ATTICA) << "BaseJob::dataFinished" << newUrl;
+        // qCDebug(ATTICA) << "BaseJob::dataFinished" << newUrl;
         QNetworkRequest request = d->m_reply->request();
         QNetworkAccessManager::Operation operation = d->m_reply->operation();
         if (newUrl.isValid() && operation == QNetworkAccessManager::GetOperation) {
@@ -110,21 +98,22 @@ void BaseJob::dataFinished()
         }
     }
 
-    if (!error) {
+    if (error) {
+        d->m_metadata.setError(Metadata::NetworkError);
+        d->m_metadata.setStatusCode(d->m_reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt());
+        d->m_metadata.setStatusString(d->m_reply->errorString());
+        d->m_metadata.setHeaders(d->m_reply->rawHeaderPairs());
+    } else {
         QByteArray data = d->m_reply->readAll();
-        //qCDebug(ATTICA) << "XML Returned:\n" << data;
+        // qCDebug(ATTICA) << "XML Returned:\n" << data;
         parse(QString::fromUtf8(data.constData()));
-        if (d->m_metadata.statusCode() == 100) {
+        if (d->m_metadata.statusCode() >= 100 && d->m_metadata.statusCode() < 200) {
             d->m_metadata.setError(Metadata::NoError);
         } else {
             d->m_metadata.setError(Metadata::OcsError);
         }
-    } else {
-        d->m_metadata.setError(Metadata::NetworkError);
-        d->m_metadata.setStatusCode(d->m_reply->error());
-        d->m_metadata.setStatusString(d->m_reply->errorString());
     }
-    emit finished(this);
+    Q_EMIT finished(this);
 
     d->m_reply->deleteLater();
     deleteLater();
@@ -132,30 +121,35 @@ void BaseJob::dataFinished()
 
 void BaseJob::start()
 {
-    QTimer::singleShot(0, this, &BaseJob::doWork);
+    if (!d->started) {
+        d->started = true;
+        QTimer::singleShot(0, this, &BaseJob::doWork);
+    }
 }
 
 void BaseJob::doWork()
 {
+    if (d->aborted) {
+        return;
+    }
     d->m_reply = executeRequest();
     qCDebug(ATTICA) << "executing" << Utils::toString(d->m_reply->operation()) << "request for" << d->m_reply->url();
     connect(d->m_reply, &QNetworkReply::finished, this, &BaseJob::dataFinished);
-    connect(d->m_reply->manager(), &QNetworkAccessManager::authenticationRequired,
-            this, &BaseJob::authenticationRequired);
-    connect(d->m_reply, static_cast<void(QNetworkReply::*)(QNetworkReply::NetworkError)>(&QNetworkReply::error),
-      [](QNetworkReply::NetworkError code){
-          qCDebug(ATTICA) << "error found" << code;
+    connect(d->m_reply->manager(), &QNetworkAccessManager::authenticationRequired, this, &BaseJob::authenticationRequired);
+    connect(d->m_reply, &QNetworkReply::errorOccurred, [](QNetworkReply::NetworkError code) {
+        qCDebug(ATTICA) << "error found" << code;
     });
 }
 
 void BaseJob::authenticationRequired(QNetworkReply *reply, QAuthenticator *auth)
 {
-    auth->setUser(reply->request().attribute((QNetworkRequest::Attribute) BaseJob::UserAttribute).toString());
-    auth->setPassword(reply->request().attribute((QNetworkRequest::Attribute) BaseJob::PasswordAttribute).toString());
+    auth->setUser(reply->request().attribute((QNetworkRequest::Attribute)BaseJob::UserAttribute).toString());
+    auth->setPassword(reply->request().attribute((QNetworkRequest::Attribute)BaseJob::PasswordAttribute).toString());
 }
 
 void BaseJob::abort()
 {
+    d->aborted = true;
     if (d->m_reply) {
         d->m_reply->abort();
         d->m_reply->deleteLater();
@@ -163,6 +157,11 @@ void BaseJob::abort()
     deleteLater();
 }
 
+bool BaseJob::isAborted() const
+{
+    return d->aborted;
+}
+
 PlatformDependent *BaseJob::internals()
 {
     return d->m_internals;
@@ -177,4 +176,3 @@ void BaseJob::setMetadata(const Attica::Metadata &data) const
 {
     d->m_metadata = data;
 }
-
